==> node-thrift-service-1.0.0.tgz: package/lib/transport.js <==
/**
 * inner thrift msg transport
 *
 * @author zzxun <xtkml.g@gmail.com>
 */
'use strict';

/**
 * module dependencies
 */
const _ = require('lodash');
const Promises = require('bluebird');
const ThriftMsg = require('../gen-nodejs/Message'),
      ttypes    = require('../gen-nodejs/msg_types'),
      utils     = require('./util');

const ThriftCallingException = ttypes.ThriftCallingException;
const RMsg = ttypes.RMsg;

exports.Processor = ThriftMsg;

/**
 * call handler
 *
 * @param cmsg
 * @param service
 * @param callback
 */
function callHandler(cmsg, service, callback) {

  // get params
  let base   = cmsg.base,
      caller = cmsg.call;
  // caller.
  if (service && service.origin[caller.action]) {
    // check permission
    if (service.actions && !service.actions[caller.action]) {
      callback(new ThriftCallingException({
        err    : 'action error',
        message: 'aciton \'' + caller.action + '\' forbidden'
      }), null);
    }
    let median = service.origin[caller.action].apply(null, JSON.parse(caller.params));
    // maybe promise
    if (_.isFunction(median.then) && _.isFunction(median.catch)) {
      median.then((result) => {
          let rmsg = new RMsg({
            base: base,
            res : JSON.stringify({
              result: result
            })
          });
          callback(null, rmsg);
        })
        .catch((err) => {
          callback(new ThriftCallingException({err: err, message: err.message}), null);
        });
    }
    // sync
    else {
      callback(null, new RMsg({
        base: base,
        res : JSON.stringify({
          result: median
        })
      }));
    }
  } else {
    // no handler
    callback(new ThriftCallingException({
      err    : 'action error',
      message: 'cannot find service \'' + caller.name + '\' or action \'' + caller.action + '\''
    }), null);
  }

}

exports.callingHandler = callHandler;

/**
 * client call remote service
 * 1. sender, ThriftClient._id
 * 2. client, thrift.createClient
 * 3. alias, action, params
 *
 * @returns {bluebird|exports|module.exports}
 */
function doCall() {
  let sender = arguments[0],
      client = arguments[1],
      alias  = arguments[2],
      action = arguments[3],
      params = arguments[4];

  // msg
  let cmsg = new ttypes.CMsg({
    base: new ttypes.Base({sender: sender, id: utils.randStr()}),
    call: new ttypes.Call({name: alias, action: action, params: JSON.stringify(params)})
  });

  return new Promises((resolve, reject) => {
    client.call(cmsg, function (err, rmsg) {
      if (err) {
        return reject(err);
      }
      return resolve(rmsg);
    });
  });
}

exports.doCall = doCall;

==> node-thrift-service-1.1.0.tgz: package/lib/transport.js <==
/**
 * inner thrift msg transport
 *
 * @author zzxun <xtkml.g@gmail.com>
 */
'use strict';

/**
 * module dependencies
 */
const _ = require('lodash');
const Promises = require('bluebird');
const ThriftMsg = require('../gen-nodejs/Message'),
      ttypes    = require('../gen-nodejs/msg_types'),
      utils     = require('./util');

const ThriftCallingException = ttypes.ThriftCallingException;
const RMsg = ttypes.RMsg;

exports.Processor = ThriftMsg;

/**
 * call handler
 *
 * @param cmsg
 * @param service
 * @param callback
 */
function callHandler(cmsg, service, callback) {

  // get params
  let base   = cmsg.base,
      caller = cmsg.call;
  // caller.
  if (service && service.origin[caller.action]) {
    // check permission
    if (service.actions && !service.actions[caller.action]) {
      callback(new ThriftCallingException({
        err    : 'action error',
        message: 'aciton \'' + caller.action + '\' forbidden'
      }), null);
    }
    let median = service.origin[caller.action].apply(null, JSON.parse(caller.params));
    // maybe promise
    if (_.isFunction(median.then) && _.isFunction(median.catch)) {
      median.then((result) => {
          let rmsg = new RMsg({
            base: base,
            res : JSON.stringify({
              result: result
            })
          });
          callback(null, rmsg);
        })
        .catch((err) => {
          callback(new ThriftCallingException({err: err, message: err.message}), null);
        });
    }
    // sync
    else {
      callback(null, new RMsg({
        base: base,
        res : JSON.stringify({
          result: median
        })
      }));
    }
  } else {
    // no handler
    callback(new ThriftCallingException({
      err    : 'action error',
      message: 'cannot find service \'' + caller.name + '\' or action \'' + caller.action + '\''
    }), null);
  }

}

exports.callingHandler = callHandler;

/**
 * client call remote service
 * 1. sender, ThriftClient._id
 * 2. client, thrift.createClient
 * 3. alias, action, params
 *
 * @returns {bluebird|exports|module.exports}
 */
function doCall() {
  let sender = arguments[0],
      client = arguments[1],
      alias  = arguments[2],
      action = arguments[3],
      params = arguments[4];

  // msg
  let cmsg = new ttypes.CMsg({
    base: new ttypes.Base({sender: sender, id: utils.randStr()}),
    call: new ttypes.Call({name: alias, action: action, params: JSON.stringify(params)})
  });

  return new Promises((resolve, reject) => {
    client.call(cmsg, (err, rmsg) => {
      if (err) {
        return reject(err);
      }
      return resolve(rmsg);
    });
  });
}

exports.doCall = doCall;
